import java.io.*;
import java.net.*;

public class ServerInfo {
    private int               numThreads;
    private static int        maxNumThreads = 10;
    private ServerSecuredComm ssc;
    private String            id;
    private String            port;

    public ServerInfo(Socket s, ServerKeyPair skp, String i, ServerSocket ss) {
        numThreads = 0;
        ssc = new ServerSecuredComm(s, skp);
        id = i;
        port = Integer.toString(ss.getLocalPort());
    }

    public boolean init() {
        try {
            if (!ssc.init_()) {
                System.err.println("Server: Key verification failed");
                System.exit(1);
            }

            // send port info to Coordinator
            ssc.setupMessage(6, port);
            ssc.send();

            // send Server ID to Coordinator
            ssc.setupMessage(7, id);
            ssc.send();

            // send total capacity to Coordinator
            ssc.setupMessage(9, Integer.toString(this.getMaxNumThreads()));
            ssc.send();

            // send availability to Coordinator
            ssc.setupMessage(8, Integer.toString(this.getMaxNumThreads() - this.getNumThreads()));
            ssc.send();

            return true;
        } catch (Exception ex) {
            System.err.println("ServerInfo.init() Exception-> " + ex);
            return false;
        }
    }

    public synchronized int getMaxNumThreads() {
        return this.maxNumThreads;
    }

    public synchronized int getNumThreads() {
        return this.numThreads;
    }

    public synchronized boolean increase() {
        if (++numThreads > maxNumThreads) {
            numThreads--;
            return false;
        } 
        // send availability to Coordinator
        try {
            ssc.setupMessage(8, Integer.toString(this.getMaxNumThreads() - this.getNumThreads()));
            ssc.send();
        } catch (Exception ex) {
            System.err.println("ServerInfo.increase() Exception-> " + ex);
        }
        return true;
    }

    public synchronized void decrease() {
        numThreads--;
        // send availability to Coordinator
        try {
            ssc.setupMessage(8, Integer.toString(this.getMaxNumThreads() - this.getNumThreads()));
            ssc.send();
        } catch (Exception ex) {
            System.err.println("ServerInfo.decrease() Exception-> " + ex);
        }
    }
}

